/**
Klasse des dreidimensionalen Darstellungsraums.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version.
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die
Free Software Foundation, Inc.,
59 Temple Place,
Suite 330,
Boston, MA 02111-1307,
USA.
*/
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.text.*;
import java.awt.font.*;
import java.util.*;

public class Disp3D {
private Graphics2D gc         = null;             // graphicContext
private Dimension  size       = null;             // Dimension des Displays
private Color      paintColor = Color.black;      // Zeichenfarbe
private Color      backgColor = Color.white;      // Hintergrungfarbe
private double[][] tmat       = Matrix.unit( 4);  // Transformationsmatrix
private double     xtrans     =   0;              // Translation entlang der X-Achse
private double     ytrans     =   0;              // Translation entlang Y-Achse
private double     ztrans     =   0;              // Translation entlang Z-Achse
public double      xscale     = 300;              // 340 Skalierung der X-Achse
public double      yscale     = 200;              // 190 Skalierung der Y-Achse
public double      zscale     = 300;              // 400 Skalierung der Z-Achse
public double      xrot       =  20;              // Rotation um X-Achse 20
public double      yrot       =  40;              // Rotation um Z-Achse 15
public double      zrot       =   0;              // Rotation um Z-Achse 0
public double      xMax       = Double.MIN_VALUE; // maximaler X-Wert
public double      yMax       = Double.MIN_VALUE; // maximaler Y-Wert
public double      zMax       = Double.MIN_VALUE; // maximaler Z-Wert

public boolean     dots       = false;            // Punkte / kreuze darstellen (true = Kreuze)
public boolean     noRed      = false;
public boolean     noGreen    = false;
public boolean     noBlue     = false;

public Plane panexy = new Plane();
public Plane panexz = new Plane();
public Plane paneyz = new Plane();

 /**
 Bereitstellung einer Matrix fr die Translation
 @return mat: Matrix als (double)Array[][]
 */
 public double[][] trans() {
  return new double[][] {
                         {1, 0, 0, xtrans},
                         {0, 1, 0, ytrans},
                         {0, 0, 1, ztrans},
                         {0, 0, 0,  1}
                        };
 }

 /**
 Bereitstellung einer Matrix fr die Skalierung
 @return mat: Matrix als (double)Array[][]
 */
 public double[][] scale() {
 return new double[][] {
                        {xscale, 0,      0,      0},
                        {0,      yscale, 0,      0},
                        {0,      0,      zscale, 0},
                        {0,      0,      0,      1}
                       };
 }

 /**
 Bereitstellung einer Matrix fr Drehung um X-Achse
 @param alpha: Winkel in Grad
 @return mat: Matrix als (double)Array[][]
 */
 public double[][] rotx( double alpha) {
 double phi = Math.toRadians( alpha);
 double sin = Math.sin( phi);
 double cos = Math.cos( phi);
 double[][] mat = {
                   {1,   0,    0, 0},
                   {0, cos, -sin, 0},
                   {0, sin,  cos, 0},
                   {0,   0,    0, 1}
                  };
  return mat;
 }

 /**
 Bereitstellung einer Matrix fr Drehung um Y-Achse
 @param alpha: Winkel in Grad
 @return mat: Matrix als (double)Array[][]
 */
 public double[][] roty( double alpha) {
 double phi = Math.toRadians( alpha);
 double sin = Math.sin( phi);
 double cos = Math.cos( phi);
 double[][] mat = {
                   { cos, 0, sin, 0},
                   {   0, 1,   0, 0},
                   {-sin, 0, cos, 0},
                   {   0, 0,   0, 1}
                  };
  return mat;
 }

 /**
 Bereitstellung einer Matrix fr Drehung um Z-Achse
 @param alpha: Winkel in Grad
 @return mat: Matrix als (double)Array[][]
 */
 public double[][] rotz( double alpha) {
 double phi = Math.toRadians( alpha);
 double sin = Math.sin( phi);
 double cos = Math.cos( phi);
 double[][] mat = {
                   {cos, -sin, 0, 0},
                   {sin,  cos, 0, 0},
                   {  0,    0, 1, 0},
                   {  0,    0, 0, 1}
                  };
  return mat;
 }

 /**
 Bereitstellung einer Matrix fr Drehungen um X-,Y- und Z-Achse
 @param alpha: Rotationswinkel um X-Achse in Grad
 @param beta:  Rotationswinkel um X-Achse in Grad
 @param gamma: Rotationswinkel um X-Achse in Grad
 @return mat: Matrix der Rotationen in double[][]
 */
 public double[][] rotate() {
 double phia = Math.toRadians( xrot);
 double sina = Math.sin( phia);
 double cosa = Math.cos( phia);
 double phib = Math.toRadians( yrot);
 double sinb = Math.sin( phib);
 double cosb = Math.cos( phib);
 double phic = Math.toRadians( zrot);
 double sinc = Math.sin( phic);
 double cosc = Math.cos( phic);

 double[][] mat =                       {
                                         {1, 0,    0,     0},
                                         {0, cosa, -sina, 0},
                                         {0, sina,  cosa, 0},
                                         {0, 0,    0,     1}
                                        };

  mat = Matrix.multiply( mat,
                         new double[][] {
                                         { cosb, 0, sinb, 0},
                                         { 0,    1, 0,    0},
                                         {-sinb, 0, cosb, 0},
                                         { 0,    0, 0,    1}
                                        }
                       );

  mat = Matrix.multiply( mat,
                         new double[][] {
                                         {cosc, -sinc, 0, 0},
                                         {sinc,  cosc, 0, 0},
                                         {0,     0,    1, 0},
                                         {0,     0,    0, 1}
                                        }
                       );
  return mat;
 }

 /**
 Projektion eines 3D-Punktes auf die Bildebene.
 @param point: zu projetierender 3D-Punkt.
 @return pt: projektierter Punkt als Point2D.
 */
 public Point2D proj( Point3D point) {
 Point2D  pt = new Point2D.Double();
 double[] tp = {point.x, point.y, point.z, 1};

  tp = Matrix.multiply( tmat, tp);
  pt.setLocation( tp[0]+size.getWidth()/2, -tp[1]+size.getHeight()/2);
  return pt;
 }

 /**
 Zeichnen einer durch zwei 3D-Punkte begrenzten geraden Linie,
 @param point0: Anfangspunkt.
 @param point1: Endpunkt.
 */
 public void line( Point3D point0, Point3D point1) {
 Line2D ln = new Line2D.Double( proj( point0), proj( point1));
  gc.draw( ln);
 }

 /**
 Zeichnen einer geraden Linie bestimmter Lnge und 'X'/'Y'/'Z'-Richtung,
 @param point0: Anfangspunkt.
 @param dis: Richtung.
 @param len: Lnge.
 */
 public void line( Point3D point0, char dir, double len) {
 Point3D point1 = null;
  switch( dir) {
   case 'X': point1 = new Point3D( point0.x+len/xscale/2.0, point0.y, point0.z); break;
   case 'Y': point1 = new Point3D( point0.x, point0.y+len/yscale/2.0, point0.z); break;
   case 'Z': point1 = new Point3D( point0.x, point0.y, point0.z+len/zscale/2.0); break;
  }
  line( point0, point1);
 }

 /**
 Linie mit 2D Pfeilspitze am Ende zeichnen. Richtung zeigt zum Linienende.
 @param point0: Anfangspunkt.
 @param point1: Endpunkt.
 */
 public void arrow( Point3D point0, Point3D point1) {
 Line2D      line = new Line2D.Double( proj( point0), proj( point1));
 GeneralPath arr  = new GeneralPath();
 double      xend = line.getP2().getX();
 double      yend = line.getP2().getY();
 double      dx   = xend-line.getP1().getX();
 double      phi;

  gc.draw( line);

  /*---------------------------------*/
  /* Aufbau der Pfeilspitze (nur 2D) */
  /*---------------------------------*/

  arr.moveTo(   0f+(float)xend,  0f+(float)yend);
  arr.lineTo( -20f+(float)xend, -5f+(float)yend);
  arr.lineTo( -20f+(float)xend,  5f+(float)yend);
  arr.lineTo(   0f+(float)xend,  0f+(float)yend);

  /*-----------------------------------*/
  /* Rotation der Pfeilspitze (nur 2D) */
  /*-----------------------------------*/

  if  (dx>=0)
       phi = Math.atan( (yend-line.getP1().getY())/dx);
  else phi = Math.atan( (yend-line.getP1().getY())/dx)+Math.PI;
  AffineTransform rotation = new AffineTransform().getRotateInstance( phi, (float)xend, (float)yend);
  gc.draw( rotation.createTransformedShape( arr));
 }

 /**
 Darstellung eines Kreuzes im Drastellungsraum
 */
 public void cross( Point3D at, double size) {
 Point3D at1x = new Point3D( at.x-size/xscale/2.0, at.y,                 at.z);
 Point3D at1y = new Point3D( at.x,                 at.y-size/yscale/2.0, at.z);
 Point3D at1z = new Point3D( at.x,                 at.y,                 at.z-size/zscale/2.0);
 Point3D at2x = new Point3D( at.x+size/xscale/2.0, at.y,                 at.z);
 Point3D at2y = new Point3D( at.x,                 at.y+size/yscale/2.0, at.z);
 Point3D at2z = new Point3D( at.x,                 at.y,                 at.z+size/zscale/2.0);
  line( at1x, at2x);
  line( at1y, at2y);
  line( at1z, at2z);
 }
 public void crossXY( Point3D at, double size) {
 Point3D at1x = new Point3D( at.x-size/xscale/2.0, at.y,                 at.z);
 Point3D at1y = new Point3D( at.x,                 at.y-size/yscale/2.0, at.z);
 Point3D at2x = new Point3D( at.x+size/xscale/2.0, at.y,                 at.z);
 Point3D at2y = new Point3D( at.x,                 at.y+size/yscale/2.0, at.z);
  line( at1x, at2x);
  line( at1y, at2y);
 }
 public void crossXZ( Point3D at, double size) {
 Point3D at1x = new Point3D( at.x-size/xscale/2.0, at.y, at.z);
 Point3D at1z = new Point3D( at.x,                 at.y, at.z-size/zscale/2.0);
 Point3D at2x = new Point3D( at.x+size/xscale/2.0, at.y, at.z);
 Point3D at2z = new Point3D( at.x,                 at.y, at.z+size/zscale/2.0);
  line( at1x, at2x);
  line( at1z, at2z);
 }
 public void crossYZ( Point3D at, double size) {
 Point3D at1y = new Point3D( at.x, at.y-size/yscale/2.0, at.z);
 Point3D at1z = new Point3D( at.x, at.y,                 at.z-size/zscale/2.0);
 Point3D at2y = new Point3D( at.x, at.y+size/yscale/2.0, at.z);
 Point3D at2z = new Point3D( at.x, at.y,                 at.z+size/zscale/2.0);
  line( at1y, at2y);
  line( at1z, at2z);
 }

 public void dot( Point3D at) {
  line( at, 'X', 1);
 }
 public void dotXY( Point3D at) {
  line( new Point3D( at.x, at.y, -1.0), 'X', 1);
 }
 public void dotXZ( Point3D at) {
  line( new Point3D( at.x, -1.0, at.z), 'X', 1);
 }
 public void dotYZ( Point3D at) {
  line( new Point3D( 1.0,  at.y, at.z), 'X', 1);
 }

 /**
 Darstellung eines reellen 3D-Punktes im Raum und seiner Projektionen auf die Ebenen
 @param at: 3D-Punkt im Rau
 */
 public void show( Point3D at) {
  if (dots)
     {
      if (!noRed)
         {
          getGraphicContext().setColor( Color.red.darker());   // Projektion auf XY-Ebene (rot)
          dotXY( at);
         }
      if (!noGreen)
         {
          getGraphicContext().setColor( Color.green.darker()); // Projektion auf YZ-Ebene (grn)
          dotYZ( at);
         }
      if (!noBlue)
         {
          getGraphicContext().setColor( Color.blue.darker());  // Projektion auf XZ-Ebene (blau)
          dotXZ( at);
         }
      getGraphicContext().setColor( Color.black);
      dot( at);
      return;
     }
  if (!noRed)
     {
      getGraphicContext().setColor( Color.red.darker());   // Projektion auf XY-Ebene (rot)
      crossXY( new Point3D( at.x, at.y, -1.0), 2);
     }
  if (!noGreen)
     {
      getGraphicContext().setColor( Color.green.darker()); // Projektion auf YZ-Ebene (grn)
      crossYZ( new Point3D( 1.0,  at.y, at.z), 2);
     }
  if (!noBlue)
     {
      getGraphicContext().setColor( Color.blue.darker());  // Projektion auf XZ-Ebene (blau)
      crossXZ( new Point3D( at.x, -1.0, at.z), 2);
     }
  getGraphicContext().setColor( Color.black);
  cross( at, 10);                                      // Projektion in den Darstellungsraum
 }

 /**
 Projektion der Ebene in den Darstellungsraum
 @param disp: der Darstellungsraum
 */
 public void project( Plane plane) {
  for (int i=0; i<4; i++)
      {
       switch (plane.how) {
        case Plane.YZ: plane.p[i] = proj( new Point3D( plane.shift, (float)plane.p[i].getX(), (float)plane.p[i].getY())); break;
        case Plane.XZ: plane.p[i] = proj( new Point3D( (float)plane.p[i].getX(), plane.shift, (float)plane.p[i].getY())); break;
        case Plane.XY: plane.p[i] = proj( new Point3D( (float)plane.p[i].getX(), (float)plane.p[i].getY(), plane.shift)); break;
       }
      }
 }

 /**
 Darstellung einer Ebene im Raum
 */
 public void showPlane( Plane plane, Rectangle2D rect) {
 Composite      cm = null; // hlt CompositeMode
 Paint          pm = null; // hlt PaintMode

  plane.setRectangle( rect);
  project (plane);

 AlphaComposite ac = AlphaComposite.getInstance( AlphaComposite.SRC_OVER, plane.trans);
 GradientPaint  gp = new GradientPaint( (float)plane.p[plane.p0].getX(), (float)plane.p[plane.p0].getY(), plane.start,
                                        (float)plane.p[plane.p1].getX(), (float)plane.p[plane.p1].getY(), plane.stop);

  cm = gc.getComposite(); // aktuellen CompositeMode sichern
  pm = gc.getPaint();     // aktuellen PaintMode     sichern

  gc.setComposite( ac);
  gc.setPaint( gp);

 GeneralPath path = new GeneralPath();
  path.moveTo( (float)plane.p[0].getX(), (float)plane.p[0].getY());
  path.lineTo( (float)plane.p[1].getX(), (float)plane.p[1].getY());
  path.lineTo( (float)plane.p[2].getX(), (float)plane.p[2].getY());
  path.lineTo( (float)plane.p[3].getX(), (float)plane.p[3].getY());
  path.closePath();
  gc.fill( path);
  gc.draw( path);

  gc.setPaint( pm);     // wiederherstellen des PaintNode
  gc.setComposite( cm); // wiederherstellen des CompositeNode
 }

 /**
 Text an einem 3D-Punkt ausgeben
 @param at: Position in 3D.
 @param text: auszugebender Text
 @param xrel: Faktor fr horizontale Verschiebung, bezogen auf die grafische Textlnge
 @param xrel: Faktor fr vertikale Verschiebung, bezogen auf die grafische Texthhe
 */
 public void textAt( Point3D at, String text, double xrel, double yrel) {
 Font              font    = getGraphicContext().getFont();
 FontRenderContext frc     = getGraphicContext().getFontRenderContext();

 TextLayout layout  = new TextLayout( text, font, frc);
 Rectangle2D bounds = layout.getBlackBoxBounds( 0 , layout.getCharacterCount()).getBounds();
 Point2D     dpos   = proj( at);

  layout.draw( getGraphicContext(),
               (float)(dpos.getX()+xrel*bounds.getWidth()),
               (float)(dpos.getY()+yrel*bounds.getHeight()));
 }

 /**
 Sklaierung (Wert, Strich) an bestimmter Position anzeigen
 @param ax: die zu skalierenden Achse.
 @param dir: Richtung des Skalierungsstrichs.
 @param val: Wert des Skalierungsstrichs.
 @paran at: Anfangspunkt der zu skalierenden Achse.
 @param frm: numerisches Format der Wertes.
 @param len: Lnge des Skalierungsstrichs.
 */
 public void scaleValue( char ax, char dir, double val, Point3D at, String frm, double len) {
 DecimalFormat df    = new DecimalFormat( frm);
 String        form  = df.format( val);
 Point3D       sc  = null;

  switch( ax) {
   case 'X':
            sc    = new Point3D( at.x+2.0*(val/xMax), at.y, at.z);
            line( sc, dir, len);
            textAt( new Point3D( sc.x, sc.y, sc.z+len/zscale/2.0), form, -0.5, 1.2);
            return;
   case 'Y':
            sc    = new Point3D( at.x, at.y+2.0*(val/yMax), at.z);
            line( sc, dir, len);
            textAt( new Point3D( sc.x+len/xscale/2.0, sc.y, sc.z), form, -1, 0.5);
            return;
   case 'Z':
            sc    = new Point3D( at.x, at.y, at.z-2.0*(val/zMax));
            line( sc, dir, len);
            textAt( new Point3D( sc.x+len/xscale/2.0, sc.y, sc.z), form, -1, 0.5);
  }
 }

 /**
 Skalierung einer bestimmten Achse in bestimmte Ruchtung
 @param ax zu beshriftende Achse
 @param dir Richtung der Beschriftung
 @param form Zahlenformat der Beschriftung
 @param num Anzahl der Unterteilungen
 @param at Anfangspunkt
 @param dist Entfernung der Beschriftung von der Achse
 */
 public void scale( char ax, char dir, String form, int num, Point3D at, double dist) {
 double max = 0;

  switch (ax) {
   case 'X': max = xMax; break;
   case 'Y': max = yMax; break;
   case 'Z': max = zMax; break;
  }
  for (int i = 0; i <= num; i++)
      scaleValue( ax, dir, max*(double)i/(double)num, at, form, dist);
 }

 /**
 Berechnung der maximalen Werte der Koordinaten
 @param set Wertemenge
 @param low Intervall (true = [0, 1])
 */
 public void maxValues( Vector set, boolean low) {
 Element elm = null;         // Element der Menge
 double  val = 0.0;          // Wert eines Elements

  xMax = Double.MIN_VALUE; // maximaler X-Wert
  yMax = Double.MIN_VALUE; // maximaler Y-Wert
  zMax = Double.MIN_VALUE; // maximaler Z-Wert

  for (int i = 0; i < set.size(); i++)
      {
       elm = (Element)set.elementAt( i);
       if  (!low)
            if   (elm.z < 1) continue;
            else {
                  val = (double)elm.n / (double)elm.z;
                  if (zMax < (double)elm.n) zMax = (double)elm.n;
                  if (xMax < (double)elm.z) xMax = (double)elm.z;
                 }
       else {
             val = (double)elm.z / (double)elm.n;
             if (zMax < (double)elm.z) zMax = (double)elm.z;
             if (xMax < (double)elm.n) xMax = (double)elm.n;
            }
       if (yMax < val) yMax = val;
      }
 }

 /**
 Initialisierung der Transformationsmatrix
 */
 public void init() {
  tmat = Matrix.unit( 4);                  // Initialisierung
  tmat = Matrix.multiply( tmat, trans());  // Verschiebung
  tmat = Matrix.multiply( tmat, rotate()); // Rotation
  tmat = Matrix.multiply( tmat, scale());  // Skalierung
 }

/**
 GraphicContext referenzieren
 @param g grafischer Context
 */

 public void setGraphicContext( Graphics2D g) {
  gc = g;
 }

 /**
 GraphicContext bergeben
 @return grafischer Context
 */

 public Graphics2D getGraphicContext() {
  return gc;
 }

 /**
 Dimensionierung des GraphicContext referenzieren
 @param dim Dimension des referenzierten GraphicContext
 */

 public void setSize( Dimension dim) {
  size = dim;
 }

 /**
 Dimensionierung des GraphicContext bergeben
 @return Dimension des referenzierten GraphicContext
 */

 public Dimension getSize() {
  return size;
 }

 /**
 Zeichenfarbe festlegen.
 @param color: neue Zeichenfarbe
 @see getPaintColor()
 */
 public void setPaintColor( Color color) {
  paintColor = color;
  gc.setColor( paintColor);
 }

 /**
 Zeichenfarbe bergeben.
 @return aktuelle Zeichenfarbe
 @see setPaintColor( col)
 */
 public Color getPaintColor() {
  return paintColor;
 }


 public void paintPlanes() {
 Rectangle2D plane = new Rectangle2D.Double( -1, 1, 2, 2);

  if (!noRed)
     {
      panexy.setOrientation( Plane.XY);
      panexy.p0 = 0;              // Ecke mit kleinen Werten
      panexy.p1 = 2;              // Ecke mit grossen Werten
      panexy.stop  = Color.red;   // erste  Verlaufsfarbe der Ebene
      panexy.start = Color.white; // zweite Verlaufsfarbe der Ebene
      panexy.shift = -1.0;        // Position der Ebene im Darstellungsraum
      showPlane( panexy, plane);  // XY-Ebene darstellen
     }
  if (!noBlue)
     {
      panexz.setOrientation( Plane.XZ);
      panexz.p0 = 0;              // Ecke mit kleinen Werten
      panexz.p1 = 2;              // Ecke mit grossen Werten
      panexz.stop  = Color.blue;  // erste  Verlaufsfarbe der Ebene
      panexz.start = Color.white; // zweite Verlaufsfarbe der Ebene
      panexz.shift = -1.0;        // Position der Ebene im Darstellungsraum
      showPlane( panexz, plane);  // XZ-Ebene darstellen
     }
  if (!noGreen)
     {
      paneyz.setOrientation( Plane.YZ);
      paneyz.p0 = 1;              // Ecke mit kleinen Werten
      paneyz.p1 = 3;              // Ecke mit grossen Werten
      paneyz.stop  = Color.green; // erste  Verlaufsfarbe der Ebene
      paneyz.start = Color.white; // zweite Verlaufsfarbe der Ebene
      paneyz.shift = 1.0;         // Position der Ebene im Darstellungsraum
      showPlane( paneyz, plane);  // YZ-Ebene darstellen
     }
 }

 public void paintRoom() {
  arrow( new Point3D( -1,    -1,     1),    // X-Achse (Nenner)
         new Point3D(  1.10, -1,     1));
  arrow( new Point3D( -1,    -1,    -1),    // Y-Achse (Werte)
         new Point3D( -1,     1.14, -1));
  arrow( new Point3D( -1,    -1,     1),    // Z-Achse (Zhler)
         new Point3D( -1,    -1,    -1.2));

  if (!(noRed || noGreen || noBlue))
     {
      getGraphicContext().setColor( Color.lightGray);
      line( new Point3D(  1,  1,  1),  // Z-Achse (Begrenzung)
            new Point3D(  1,  1, -1));
      line( new Point3D(  1, -1,  1),  // Y-Achse (Begrenzung)
            new Point3D(  1,  1,  1));
      line( new Point3D( -1,  1, -1),  // X-Achse (Begrenzung)
            new Point3D(  1,  1, -1));
     }
 }

 public void paintAxix() {
  getGraphicContext().setColor( Color.black);
  textAt( new Point3D(  1.10, -1,     1),   "Nenner",  0, 1);
  textAt( new Point3D( -1,     1.14, -1),   "Wert",   -1, 0);
  textAt( new Point3D( -1,    -1,    -1.2), "Zhler", -1, 0);

  /*---------------------------------------*/
  /* Koordinatenachsen mit Skalen versehen */
  /*---------------------------------------*/

  scale( 'Y', 'X', "#.#", 2, new Point3D( -1.0, -1.0, -1.0), -20); // Y-Achse beschriften
  scale ('Z', 'X', "#",   3, new Point3D( -1.0, -1.0,  1.0), -20); // Z-Achse beschriften
  scale( 'X', 'Z', "#",   4, new Point3D( -1.0, -1.0,  1.0),  40); // X-Achse beschriften
 }

 /**
 Wandlung eines Elements der Folge in einen 3D-Punkt.
 @param elm Element der Folge
 @return 3D-Punkt
 */
 public Point3D elmToPoint( Element elm, boolean low) {
  if (low)
      return new Point3D( -1.0+2.0*(double)elm.n/xMax,
                          -1.0+2.0*(double)elm.z / (double)elm.n / yMax,
                           1.0-2.0*(double)elm.z/zMax);

  return new Point3D( -1.0+2.0*(double)elm.z/xMax,
                      -1.0+2.0*(double)elm.n / (double)elm.z / yMax,
                       1.0-2.0*(double)elm.n/zMax);
 }

 /**
 Wertemenge darstellen
 @param set Wertemenge
 @param low Intervall (true = [0,1])
 @param connect Punkte verbinden (true = ja)
 */
 public void paintSet( Vector set, boolean low, boolean connect) {
 Element      elm = null; // aktuelles  Element der Menge
 Element      prv = null; // vorheriges Element der Menge
 ListIterator iterator;   // Menge (Vector) ist Liste - daher Iterator

  iterator = set.listIterator( 0);
  while (iterator.hasNext())
      {
       elm = (Element)iterator.next();
       if  (!low && elm.z < 1)
            continue;
       else show( elmToPoint( elm, low));    // Punkt darstellen

       if (connect)
          {
           if   (prv != null)
                line( elmToPoint( prv, low), // nit Vorgnger verbinden
                      elmToPoint( elm, low));
           prv = elm;
          }
      }
 }

 public void paintAll( Vector set, boolean low, boolean connect) {
  paintPlanes(); // Ebenen darstellen
  paintRoom();                          // Koordinatenachsen darstellen
  paintAxix();                          // Koordinatenachsen benennen
  paintSet( set, low, connect);         // Darstellung aller mglichen Projektionen der Zahlenmenge
 }

 public void clear() {
 System.out.println( "Clear aufgerufen: "+size);
  gc.fillRect( 0,
               0,
               (int)size.getWidth(),
               (int)size.getHeight());
 }
}
