/**
Klasse der Matrizen zur Berechnung der Projektionen.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version.
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die
Free Software Foundation, Inc.,
59 Temple Place,
Suite 330,
Boston, MA 02111-1307,
USA.
*/
class Matrix {

 /**
 Bereitstellung einer Einheitsmatrix El,l
 @param l: Lnge der Diagonalen
 @return e: Die entsprechende Einheitsmatrix
 */
 static public double[][] unit( int l) {
 double[][] e = new double[l][l];

  for (int i=0; i<l; i++) e[i][i] = 1;
  return e;
 }

 /**
 Ausgabe der Matritzenelemente Ai,j. Die Ausgabe erfolgt
 zeilenorientiert nach System.out mit einem Umbruch nach
 jeder Zeile. Zwischen zwei Elementen befinden sich drei
 Leerzeichen.
 @param a: auzugebenende Matrix als (double)Array[][]
 */
 static public void show( double[][] a) {
 int m = a.length;
 int n = a[0].length;

  for (int i=0; i<m; i++)
      {
       for (int j=0; j<n; j++)
           {
            System.out.print(a[i][j]);
            if (j<n-1) System.out.print( "   ");
           }
       System.out.println("");
      }
 }

 static public void symshow( double[][] a) {
 int m = a.length;
 int n = a[0].length;

 for (int i=0; i<m; i++)
     {
      for (int j=0; j<n; j++)
          {
           System.out.print("a");
           System.out.print(""+i+",");
           System.out.print(""+j+" = ");
           if (a[i][j]<10) System.out.print(" ");
           System.out.print(""+a[i][j]+";   ");
          }
      System.out.println("");
     }
 }

 /**
 Ausgabe eines Vektors nach System.out. Zwischen zwei
 Elementen befindet sich einKomma, gefolgt von einem Leerzeichen.
 @param v: auzugebenender Vektor als (double)Array[]
 */
 static public void showVect( double[] v) {
  for (int i=0; i<v.length; i++)
      {
       System.out.print( ""+v[i]);
       if (i<v.length-1) System.out.print( ", ");
      }
 }

 /**
 Transponieren einer Matrix.
 @param a: zu transponierende Matrix als (double)Array[][]
 @return t: die transponierte Matrix als (double)Array[][]
 */
 static public double[][] transponse( double[][] a) {
 int m = a.length;
 int n = a[0].length;
 double[][] t = new double[n][m];

 for (int i=0; i<m; i++)
      for (int j=0; j<n; j++)
           t[j][i] = a[i][j];
  return t;
 }

 /**
 Addition zweier Matritzen.
 @param a: 1. Summand ist Matrix als (double)Array[][]
 @param b: 2. Summand ist Matrix als (double)Array[][]
 @return c: Summe a+b ist Matrix als (double)Array[][]
 */
 static public double[][] add( double[][] a, double[][] b) {
 int m = a.length;
 int n = a[0].length;
 double[][] c = new double[m][n];

 for (int i=0; i<m; i++)
      for (int j=0; j<n; j++)
           c[i][j] = a[i][j] + b[i][j];
  return c;
 }

 /**
 Multiplikation zweier Matritzen.
 @param a: 1. Multiplikant ist Matrix als (double)Array[m][k]
 @param b: 2. Multiplikant ist Matrix als (double)Array[k][n]
 @return c: Produkt a*b ist Matrix als (double)Array[m][n]
 */
 static public double[][] multiply( double[][] a, double[][] b) {
 int m = a.length;
 int n = a[0].length;
 int l = b.length;
 double[][] c = new double[m][n];
 double sum;

  for (int i=0; i<m; i++)
      {
       for (int j=0; j<l; j++)
           {
            sum = 0;
            for (int k=0; k<n; k++)
                 sum += a[i][k]*b[k][j];
            c[i][j] = sum;
           }
      }
  return c;
 }

 /**
 Multiplikation einer Matrix mit einem Vektor.
 @param a: 1. Multiplikant ist Matrix als (double)Array[m][k]
 @param b: 2. Multiplikant ist Vektor als (double)Array[m]
 @return c: Produkt a*b ist Vektor als (double)Array[m]
 */
 static public double[] multiply( double[][] a, double[] b) {
 int      m = a.length;
 int      n = b.length;
 double[] c = new double[m];
 double   v;

  for (int i=0; i<n; i++)
      {
       v = 0;
       for (int j=0; j<m; j++)
           {
            v += b[j]*a[i][j];
           }
       c[i] = v;
      }
  return c;
 }

 /**
 Invertieren einer Matrix.
 mit i=Zeilenanzahl und j=Spaltenanzahl. Dabei ist i=j vorausgesetzt
 @param a: zu invertierende Matrix als (double)Array[][]
 @return b: die invertierte Matrix als (double)Array[][]
 */
 static public double[][] invert( double[][] m) {
 double     dummy;
 int        n = m[0].length;
 double[][] a = new double[n][n];
 double[][] b = new double[n][n];

 for (int k=0; k<n; k++)
     {
      for (int j=0; j<n; j++)
          {
           a[k][j] = m[k][j];
          }
      b[k][k] = 1.0;
     }

 for (int k=0; k<n; k++)
     {
      for (int j=0; j<n; j++)
          {
           b[k][j] = 0.0;
          }
      b[k][k] = 1.0;
     }

 for (int k=0; k<n; k++)
     {
      dummy = a[k][k];
      for (int j=0; j<n; j++)
          {
           a[k][j] = a[k][j] / dummy;
           b[k][j] = b[k][j] / dummy;
          }
      for (int i=0; i<n; i++)
          {
           if (i != k)
              {
               dummy = a[i][k];
               for (int j=0; j<n; j++)
                   {
                    a[i][j] = a[i][j] - dummy * a[k][j];
                    b[i][j] = b[i][j] - dummy * b[k][j];
                   }
              }
          }
     }
  return b;
 }

 /**
 Bestimmten Zeilenvektor der Matrix bergeben.
 @param i: Index der Zeile als int
 @param a: Matrix als (double)Array[][]
 @return r: der Zeilenvektor als (double)Array[]
 */
 static public double[] getRow( int i, double[][] a) {
 int n = a.length;
 double[] r = new double[n];

  for (int j=0; j<n; j++) r[j] = a[i][j];
  return r;
 }

 /**
 Bestimmten Spaltenenvektor der Matrix bergeben.
 @param j: Index der Zeile als int
 @param a: Matrix als (double)Array[][]
 @return c: der Zeilenvektor als (double)Array[]
 */
 static public double[] getColumn( int j, double[][] a) {
 int n = a.length;
 double[] c = new double[n];

  for (int i=0; i<n; i++) c[i] = a[i][j];
  return c;
 }

 /**
 Ermitteln der Hauptdiagonalen einer Matrix. Die Matrix ist ein Array[i][j]
 mit i=Zeilenanzahl und j=Spaltenanzahl. Dabei ist i=j vorausgesetzt
 @param a: Matrix als (double)Array[][]
 @return d: die Hauptdiagonale als (double)Array[]
 */
 static public double[] mainDiagonal( double[][] a) {
 int n = a.length;
 double[] d = new double[n];

  for (int i=0; i<n; i++) d[i] = a[i][i];
  return d;
 }

 /**
 Ermitteln der Nebendiagonalen einer Matrix. Die Matrix ist ein Array[i][j]
 mit i=Zeilenanzahl und j=Spaltenanzahl. Dabei ist i=j vorausgesetzt
 @param a: Matrix als (double)Array[][]
 @return d: die nebendiagonale als (double)Array[]
 */
 static public double[] subDiagonal( double[][] a) {
 int n = a.length;
 double[] d = new double[n];

  for (int i=0; i<n; i++) d[i] = a[i][n-1-i];
  return d;
 }

 /**
 Ermitteln der Spur einer Matrix. Die Matrix ist ein Array[i][j]
 mit i=Zeilenanzahl und j=Spaltenanzahl. Dabei ist i=j vorausgesetzt
 @param a: Matrix als (double)Array[][]
 @return s: die Spur als Summe aller Elemente der Haupdiagonalen.
 @see mainDiagonal( double[][] a)
 */
 static public double spur( double[][] a) {
 int n = a.length;
 double s = 0.0;

  for (int i=0; i<n; i++) s = s + a[i][i];
  return s;
 }

 static public void main( String args[]) {
/*
 double[][] a = {
                 { 1.0/3.0, 1.0/9.0,  1.0/9.0},
                 { 0.0,     1.0/3.0, -1.0/6.0},
                 {-2.0/3.0, 4.0/9.0, -1.0/18.0}
                };
  System.out.println( "original");
  show(a);
  System.out.println( "invertiert");
  show(invert(a));
*/

 double[][] a = {
                 {1, 3, 4},
                 {2, 4, 2},
                 {3, 4, 7},
                };

 double[][] b = {
                 {1, 3, 4},
                 {2, 4, 2},
                 {3, 4, 7},
                };

  System.out.println( "Matrix");
  show(a);
  System.out.print( "\nHauptdiaginale = ");
  showVect(mainDiagonal(a));
  System.out.print( "\nNebendiaginale = ");
  showVect(subDiagonal(a));
  System.out.print( "\nSpur der Matrix = ");
  System.out.print( ""+spur(a));
  System.out.print( "\nRow-0 der Matrix = ");
  showVect(getRow(0,a));
  System.out.print( "\nRow-1 der Matrix = ");
  showVect(getRow(1,a));
  System.out.print( "\nRow-2 der Matrix = ");
  showVect(getRow(2,a));
  System.out.print( "\nCol-0 der Matrix = ");
  showVect(getColumn(0,a));
  System.out.print( "\nCol-1 der Matrix = ");
  showVect(getColumn(1,a));
  System.out.print( "\nCol-2 der Matrix = ");
  showVect(getColumn(2,a));
  System.out.println( "\nTransponierte der Matrix");
  show(transponse(a));

  System.out.println( "\nAddition");
  show(add(a,b));

 double[][] u = {
                 {1, 3, 2},
                 {2, 4, 0},
                 {3, 2, 1},
                };

 double[]   v = {2,
                 1,
                 4};

 double[] w = multiply( u, v);

  System.out.println( "\nMatrix mal Vektor");

  for (int i=0; i<w.length; i++)
         System.out.println( w[i]);

 double[][] x = {
                 {1, 3},
                 {2, 4}
                };

 double[][] y = {
                 {2, 3},
                 {0, 4}
                };

  System.out.println( "\nMatrix mal Matrix");
  show( multiply( x, y));

  System.out.println( "\nEinheitsmatrix:");
  show( unit(4));

  System.out.println( "\nfertig");
 }
}
/*
<mathxmlns="http://www.w3.org/1998/Math/MathML">
  <mi>A</mi>
  <mo>=</mo>
  <mrow>
    <mo>(</mo>
    <mtable>
      <mtr>
        <mtd>
          <mn>1</mn>
        </mtd>
        <mtd>
          <mn>2</mn>
        </mtd>
        <mtd>
          <mn>3</mn>
        </mtd>
      </mtr>
      <mtr>
        <mtd>
          <mn>4</mn>
        </mtd>
        <mtd>
          <mn>5</mn>
        </mtd>
        <mtd>
          <mn>6</mn>
        </mtd>
      </mtr>
      <mtr>
        <mtd>
          <mn>7</mn>
        </mtd>
        <mtd>
          <mn>8</mn>
        </mtd>
        <mtd>
          <mn>9</mn>
        </mtd>
      </mtr>
    </mtable>
    <mo>)</mo>
  </mrow>
</math></p>
*/
