/**
Klasse des Fensterinhalts zur Benutzung des 3D-Displays.
Copyright (C) 2006, Rene Heuer, Salingtwiete 4g. (D)20535 Hamburg
eMail: heuer@exmpl.de
Dieses Programm ist freie Software. Sie können es unter den Bedingungen der
GNU General Public License, wie von der Free Software Foundation veröffentlicht,
weitergeben und/oder modifizieren, entweder gemäß Version 2 der Lizenz oder
(nach Ihrer Option) jeder späteren Version.
Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, daß es Ihnen von
Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite
Garantie der MARKTREIFE oder der VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK.
Details finden Sie in der GNU General Public License.
Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem
Programm erhalten haben. Falls nicht, schreiben Sie an die
Free Software Foundation, Inc.,
59 Temple Place,
Suite 330,
Boston, MA 02111-1307,
USA.
*/
import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import java.util.*;
import java.text.*;

public class TestPane extends JPanel {
public Disp3D  disp    = null;  // 3D-Display
public Vector  set     = null;  // darzustellende Menge
public boolean low     = true;  // true = [0..1]; false=[1..?]
public boolean connect = false; // Punkte entsprechend der Ordinalitt verbinden (true = ja)
public boolean shwax   = false; // Raumachsen einblenden (true = ja)

 /**
 CREATOR
 @param set darzustellende Menge
 */
 public TestPane( Disp3D disp , Vector set, boolean low, boolean connect) {
  this.disp    = disp;       // Referenz auf Anzeige
  this.set     = set;        // Referenz auf Wertemenge
  this.low     = low;        // Referenz auf Intervall (true = [0...1]
  this.connect = connect;    // Punkte verbinden (true = ja)
  disp.maxValues( set, low); // Maximalwerte der Koordinaten bestimmen
 }

 /**
 Koordinatenachsen des Ursprungs zeichnen
 @param disp: das 3D-Display
 @param center: der Ursprung
 */
 public void axis( Point3D center) {
  disp.arrow( center, new Point3D(  1,  0,  0)); // positive X-Achse
  disp.arrow( center, new Point3D(  0,  1,  0)); // positive Y-Achse
  disp.arrow( center, new Point3D(  0,  0,  1)); // positive Z-Achse

  disp.arrow( center, new Point3D( -1,  0,  0)); // negative X-Achse
  disp.arrow( center, new Point3D(  0, -1,  0)); // negative Y-Achse
  disp.arrow( center, new Point3D(  0,  0, -1)); // negative Z-Achse

 Point2D at;
  at = disp.proj( new Point3D(  1+0.025, 0, 0));
  disp.getGraphicContext().drawString( "X", (float)at.getX(), (float)at.getY());
  at = disp.proj( new Point3D(  0, 1+0.025, 0));
  disp.getGraphicContext().drawString( "Y", (float)at.getX(), (float)at.getY());
  at = disp.proj( new Point3D(  0, 0, 1+0.025));
  disp.getGraphicContext().drawString( "Z", (float)at.getX(), (float)at.getY());
 }

 /**
 Darstellung des gesamten 3D-Displays
 @param g: einfacher grafischer Kontext
 */
 public void paintComponent( Graphics g) {
  super.paintComponent( g);
  setBackground( new Color(0xffffee)); // COLOR="#ffffcc"
//  setBackground( Color.white);
  disp.setGraphicContext( (Graphics2D)g); // grafischen Kontext festlegen
  disp.getGraphicContext().setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
  disp.setSize( this.getSize());          // Breite und Hhe    festlegen
  disp.init();                            // Initialisierung des Dastellungsraums
  disp.setPaintColor( Color.black);       // Zeichenfarbe im Raum festlegen

  if (shwax) axis( new Point3D( 0, 0, 0)); // ggf. Raumachsen einblenden

  disp.maxValues( set, low); // Maximalwerte der Koordinaten bestimmen

  disp.paintAll( set, low, connect);
 }
}
